# -*- coding: utf-8 -*-

__version__ = '0.3.0'

from django.contrib.admin import SimpleListFilter
from django.core.exceptions import FieldDoesNotExist
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _


def isnull_filter(field_name, filter_title=None, negate=False, operator='null'):
    class HasRelatedFieldFilter(SimpleListFilter):
        parameter_name = f'{field_name}__is{operator}'
        related_field = field_name

        def __init__(self, request, field, model, model_admin):
            if filter_title:
                self.title = filter_title
            else:
                negate_str = 'not ' if negate else ''

                try:
                    related_field = model._meta.get_field(field_name)
                except FieldDoesNotExist:
                    model.objects.all().query.resolve_ref(field_name)
                    self.title = _(f"Is field '{field_name}' {negate_str}{operator}?")
                    return super(HasRelatedFieldFilter, self).__init__(request, field, model, model_admin)

                if hasattr(related_field, 'related'):
                    related_title = related_field.related.model._meta.verbose_name_plural
                    self.title = _(f"Is related '{related_title}' {negate_str}{operator}?")
                elif hasattr(related_field, 'related_model') and hasattr(related_field.related_model, "_meta"):
                    related_title = related_field.related_model._meta.verbose_name_plural
                    self.title = _(f"Is related '{related_title}' {negate_str}{operator}?")
                else:
                    related_title = related_field.name
                    self.title = _(f"Is field '{related_title}' {operator}?")
            super(HasRelatedFieldFilter, self).__init__(request, field, model, model_admin)

        def lookups(self, request, model_admin):
            return [
                ('true', _('True')),
                ('false', _('False')),
            ]

        def queryset(self, request, queryset):
            filters = Q()

            if self.value() in ('true', 'True'):
                value = True
            elif self.value() in ('false', 'False'):
                value = False
            else:
                value = None

            if value is not None:
                filters = Q(**{f"{self.related_field}__isnull": value})
                if operator == 'blank':
                    blank_filter = Q(**{f"{self.related_field}__exact": ''})
                    filters |= blank_filter if value else ~blank_filter

            if negate:
                return queryset.exclude(filters).distinct()
            else:
                return queryset.filter(filters).distinct()

    return HasRelatedFieldFilter


def isblank_filter(field_name, filter_title=None, negate=False):
    return isnull_filter(field_name, filter_title=filter_title, negate=negate, operator='blank')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-isnull-list-filter
------------

Tests for `django-isnull-list-filter` models module.
"""
import datetime

from django.core.exceptions import FieldError
from django.test import RequestFactory, TestCase

from isnull_filter import isblank_filter, isnull_filter

from model_mommy import mommy

from . import models


class TestIsNullFilter(TestCase):
    def setUp(self):
        albums1 = mommy.make(
            'Album',
            author=None,
            released=datetime.date(2017, 1, 1),
            coauthors=mommy.make(
                'Author',
                _quantity=2,
            ),
            name="Brothers in Arms",
            _quantity=2,
        )
        mommy.make(
            'Song',
            album=albums1[1],
        ),
        mommy.make(
            'Album',
            author__name='Mark Knopfler',
            released=None,
            _quantity=4,
            name="",
        )
        mommy.make(
            'Album',
            author__name='Mark Knopfler',
            released=None,
            _quantity=4,
        )
        self.factory = RequestFactory()
        self.request = self.factory.get("")

    def test_isblank_filter_name_true(self):
        f = isblank_filter('name')(self.request, {"name__isblank": "true"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 8)

    def test_filter_song_set_yes(self):
        f = isnull_filter('song_set')(self.request, {"song_set__isnull": "true"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 9)

    def test_filter_song_set_no(self):
        f = isnull_filter('song_set')(self.request, {"song_set__isnull": "false"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 1)

    def test_filter_song_set_null(self):
        f = isnull_filter('song_set')(self.request, {}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 10)

    def test_filter_song_author_related_yes(self):
        f = isnull_filter('album__author')(self.request, {"album__author__isnull": "true"}, models.Song, None)
        q = f.queryset(self.request, models.Song.objects.all())
        self.assertEquals(q.count(), 1)

    def test_filter_song_author_related_no(self):
        f = isnull_filter('album__author')(self.request, {"album__author__isnull": "true"}, models.Song, None)
        q = f.queryset(self.request, models.Song.objects.all())
        self.assertEquals(q.count(), 1)

    def test_filter_author_yes(self):
        f = isnull_filter('author')(self.request, {"author__isnull": "true"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 2)

    def test_filter_author_no(self):
        f = isnull_filter('author')(self.request, {"author__isnull": "false"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 8)

    def test_filter_author_null(self):
        f = isnull_filter('author')(self.request, {}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 10)

    def test_filter_coauthors_yes(self):
        f = isnull_filter('coauthors')(self.request, {"coauthors__isnull": "true"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 8)

    def test_filter_coauthors_no(self):
        f = isnull_filter('coauthors')(self.request, {"coauthors__isnull": "false"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 2)

    def test_filter_coauthors_null(self):
        f = isnull_filter('coauthors')(self.request, {}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 10)

    def test_filter_released_yes(self):
        f = isnull_filter('released')(self.request, {"released__isnull": "true"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 8)

    def test_filter_released_no(self):
        f = isnull_filter('released')(self.request, {"released__isnull": "false"}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 2)

    def test_filter_released_null(self):
        f = isnull_filter('released')(self.request, {}, models.Album, None)
        q = f.queryset(self.request, models.Album.objects.all())
        self.assertEquals(q.count(), 10)

    def test_filter_released_title(self):
        f = isnull_filter('released')(self.request, {}, models.Album, None)
        self.assertEquals(f.title, u"Is field 'released' null?")

    def test_filter_song_set_title(self):
        f = isnull_filter('song_set')(self.request, {}, models.Album, None)
        self.assertEquals(f.title, u"Is related 'songs' null?")

    def test_filter_author_title(self):
        f = isnull_filter('author')(self.request, {}, models.Album, None)
        self.assertEquals(f.title, u"Is related 'Authors' null?")

    def test_filter_coauthors_title(self):
        f = isnull_filter('coauthors')(self.request, {}, models.Album, None)
        self.assertEquals(f.title, u"Is related 'Authors' null?")

    def test_filter_title_overriden(self):
        f = isnull_filter('author', 'Overriden title')(self.request, {}, models.Album, None)
        self.assertEquals(f.title, "Overriden title")

    def test_filter_unexistent_field(self):
        with self.assertRaises(FieldError):
            isnull_filter('foo')(self.request, {}, models.Album, None)



from django.db import models


class Author(models.Model):
    class Meta:
        verbose_name = "Author"
        verbose_name_plural = "Authors"

    name = models.TextField()


class Album(models.Model):
    author = models.ForeignKey(
        Author,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
    )
    coauthors = models.ManyToManyField(
        Author,
        null=True,
        blank=True,
        related_name="coauthored_albums",
    )
    released = models.DateField(
        null=True,
        blank=True,
    )
    name = models.CharField(
        null=True,
        blank=True,
        max_length=255,
    )


class Song(models.Model):
    album = models.ForeignKey(
        Album,
        related_name="song_set",
        on_delete=models.CASCADE,
    )
